@@ -4,7 +4,6 @@ package dice // import "github.com/xtls/xray-core/common/dice"
 
 import (
 	"math/rand"
-	"time"
 )
 
 // Roll returns a non-negative number between 0 (inclusive) and n (exclusive).
@@ -15,6 +14,14 @@ func Roll(n int) int {
 	return rand.Intn(n)
 }
 
+// RollInt63n returns a non-negative number between 0 (inclusive) and n (exclusive).
+func RollInt63n(n int64) int64 {
+	if n == 1 {
+		return 0
+	}
+	return rand.Int63n(n)
+}
+
 // Roll returns a non-negative number between 0 (inclusive) and n (exclusive).
 func RollDeterministic(n int, seed int64) int {
 	if n == 1 {
@@ -46,7 +53,3 @@ func (dd *DeterministicDice) Roll(n int) int {
 	}
 	return dd.Intn(n)
 }
-
-func init() {
-	rand.Seed(time.Now().Unix())
-}
