@@ -2,18 +2,20 @@ package all
 
 import (
 	"github.com/xtls/xray-core/main/commands/all/api"
+	"github.com/xtls/xray-core/main/commands/all/convert"
 	"github.com/xtls/xray-core/main/commands/all/tls"
 	"github.com/xtls/xray-core/main/commands/base"
 )
 
-// go:generate go run github.com/xtls/xray-core/common/errors/errorgen
-
 func init() {
 	base.RootCommand.Commands = append(
 		base.RootCommand.Commands,
 		api.CmdAPI,
-		// cmdConvert,
+		convert.CmdConvert,
 		tls.CmdTLS,
 		cmdUUID,
+		cmdX25519,
+		cmdWG,
+		cmdMLDSA65,
 	)
 }
